const fs = require('fs');
const path = require('path');
const postcss = require('postcss');
const postcssrc = require('postcss-load-config');
const postcssJs = require('postcss-js');

const DEFAULT_THEME = '@digigov/theme-default';

/**
 * Load custom theme CSS as base styles.
 *
 * @param {object} [options={}]
 * @param {string} [options.theme] - Custom theme package name or file path.
 * @param {boolean} [options.eject] - Whether to ignore the default Digigov CSS base styles.
 * @param {boolean} [options.debug] - Whether to log debug information.
 * @returns {object} - The base styles as CSS-in-JS object.
 */
function getThemeBaseStyle(options = {}) {
  const { theme: userTheme, eject, debug } = options;

  if (!userTheme) {
    // do not load any theme
    if (eject) return;
    // load default theme
    const css = loadCss(debug, DEFAULT_THEME);
    return processCss(css);
  }

  // only load user theme
  if (eject) {
    const css = loadCss(debug, userTheme);
    return processCss(css);
  }

  // load both default and user theme
  const css = loadCss(debug, DEFAULT_THEME, userTheme);
  return processCss(css);
}

/**
 * Check if the given theme is a local file or package.
 *
 * @param {string} userTheme - File path or package name.
 * @returns {boolean} - Whether the theme is a local file.
 */
function isLocalTheme(userTheme) {
  return (
    userTheme.startsWith('./') ||
    userTheme.startsWith('../') ||
    path.isAbsolute(userTheme)
  );
}

/**
 * Load CSS from given package or file path.
 *
 * @param {boolean} debug - Whether to log debug information.
 * @param {...string} pkgOrPathList - Package name or file path.
 *
 * @returns {string} - The CSS content.
 */
function loadCss(debug, ...pkgOrPathList) {
  const includedThemes = [...pkgOrPathList];
  const css = pkgOrPathList
    .map((pkgOrPath) => {
      try {
        let cssFilePath = isLocalTheme(pkgOrPath)
          ? path.resolve(pkgOrPath)
          : require.resolve(pkgOrPath);

        if (!fs.existsSync(cssFilePath)) {
          throw new Error(`File not found: ${cssFilePath}`);
        }

        if (path.extname(cssFilePath) === '.js') {
          const dir = path.dirname(cssFilePath);
          const indexCssFile = path.join(dir, 'index.css');
          if (fs.existsSync(indexCssFile)) {
            cssFilePath = indexCssFile;
          } else {
            throw new Error('No CSS index file found');
          }
        }

        return fs.readFileSync(cssFilePath).toString();
      } catch (error) {
        console.error(`Failed to load CSS from ${pkgOrPath}.`, error);
        delete includedThemes[includedThemes.indexOf(pkgOrPath)];
        return '';
      }
    })
    .join('\n');

  if (debug && includedThemes.length > 0)
    console.log(
      '\x1b[32m%s\x1b[0m',
      `🎨 Including theme${includedThemes.length > 1 ? 's' : ''}:`,
      '\x1b[0m',
      '' + includedThemes.join(', ')
    );

  return css;
}

/**
 * Process CSS into a CSS-in-JS object.
 * @param {string} css - The CSS content to process.
 * @returns {object} - The merged CSS-in-JS object.
 */
function processCss(css) {
  try {
    const { plugins, options } = postcssrc.sync({}, __dirname);
    const processedCss = postcss(plugins).process(css, options).result.root;
    return postcssJs.objectify(processedCss);
  } catch (error) {
    console.error('Failed to process CSS.', error);
    return {};
  }
}

module.exports = getThemeBaseStyle;
